;;-----------------------------------------------
;; CADENCE June 1995  B.Kramer
;; 
;; LISTING 1: TBUILD Function
;;-----------------------------------------------
;;
;; TBUILD: create block and set global variables
;;         for building a table.
;;-----------------------------------------------
(defun C:TBUILD ( / TH CT CW UCW CLS AN CH1 CH2)
   ;; Set system variables
   (setvar "CMDECHO" 0) ;;no command echo
   (setvar "EXPERT" 4)  ;;expert mode
   ;;
   (setq TH (GETIT "\nText height" 
                   (getvar "TEXTSIZE"))
         RH (GETIT "\nRow height" 
                   (* 1.5 TH))
   )
   ;;
   ;; While operator enters title string
   (while (/= "" 
              (setq CT 
                 (getstring 1 "\nCol. Title: ")))
      ;;
      ;;Get size of text string in drawing units
      (setq CW (textbox 
                  (list 
                     (cons 1 CT) 
                     (cons 40 TH)))
            CW (+ TH (caadr CW)) ;;just width
            UCW 0.0
      )
      ;;
      ;;As operator for column width
      (while (< UCW CW)
         (setq UCW 
            (getdist 
              (strcat "\nColumn width [min. is "
                 (rtos CW) "]: ")))
         (if (null UCW) (setq UCW CW))
      )
      ;;
      ;;Add string plus width to columns list
      (setq CLS (cons (list CT UCW) CLS))
   )
   ;;
   ;; Set a marker in the drawing
   (command "_POINT" (list 0 0 0))
   (setq EN1 (entlast)
         TW (apply '+  ;;sum the list of
              (mapcar 'cadr CLS)) ;;column widths
         TP (list 0.0 (* RH -0.5) 0) ;;text point
         CH1 0 ;;top of row
         CH2 (- 0 RH) ;;bottom of row
         AN 0  ;;attribute number
   )
   ;;
   ;;draw line at bottom of row
   (make_a_line (list 0 CH2 0)
                (list TW CH2 0)
                "0")
   ;;
   ;;for each column
   (foreach COL (reverse CLS)
      ;;
      ;;draw line at start of column
      (make_a_line 
         (list (car TP) CH1 0)
         (list (car TP) CH2 0)
         "0")
      ;;
      ;;determine text insert point
      (setq TP 
              (list 
                 (+ (car TP) 
                    (/ (cadr COL) 2.0)) 
                 (cadr TP) 
                 0.0)
            AN (1+ AN)
      )
      (command "_ATTDEF" 
               ""
               (strcat "T" (itoa AN)) ;;tag name
               (car COL) ;;prompt
               (car COL) ;;default value
               "_M"      ;;mid point request
               TP        ;;mid point location
               TH        ;;text height
               0.0       ;;rotation
      )
      (setq TP ;;move TP to end of column
         (list 
            (+ 
               (car TP) 
               (/ (cadr COL) 2.0)) 
            (cadr TP)
            0.0))
   )
   ;;
   ;;draw line at end of row
   (make_a_line 
      (list (car TP) (+ (cadr TP) CH2) 0)
      (list (car TP) (- (cadr TP) CH2) 0)
      "0")
   ;;
   ;;create the block definition
   (command 
      "_BLOCK" 
      "TABLEROW" 
      (list 0 0 0))
   ;;
   ;;loop through the entities
   (setq EN EN1)
   (while (setq EN (entnext EN))
      (command EN)
   )
   (command "")
   (entdel EN1) ;;clean up marker point
   ;
   (initget 1) ;;no null input
   (setq P1 
     (getpoint 
       "\nLocate Table's upper left corner: "))
   ;
   (command "_INSERT" "TABLEROW" P1 1 1 0)
   (foreach COL CLS
        (command "") ;;use default
   )
   ;;
   ;;draw top of table line
   (make_a_line P1 (polar P1 0.0 TW) "0")
   ;;
   ;;Set Global variables
   (setq TABLE_NEXT_ROW (polar P1 (* 1.5 PI) RH)
         TABLE_ROW_SIZE RH
   )
   (prompt "\nTable ready, use ROW  to add rows.")
   (princ)

)
;;-----------------------------------------------
;; LISTING 2: ROW Function
;;-----------------------------------------------
;; ROW: append a row to the end of the table
;;      defined in TBUILD.
;;
;;-----------------------------------------------
(defun C:ROW ()
  (if (and 
        TABLE_NEXT_ROW 
        TABLE_ROW_SIZE 
        (tblsearch "BLOCK" "TABLEROW"))
    (progn
       ;;turn off att requests
       (setvar "ATTREQ" 0)
       (command 
          "_INSERT" 
            "TABLEROW" 
            TABLE_NEXT_ROW 
            1 
            1 
            0
          "_DDATTE" 
            (entlast)
       )
       (setvar "ATTREQ" 1)
       (setq TABLE_NEXT_ROW 
          (list 
            (car TABLE_NEXT_ROW)
            (- (cadr TABLE_NEXT_ROW)
               TABLE_ROW_SIZE)
            0.0))
    )
    (prompt "\nRun TBUILD first.")
  )
  (princ)
)
;------------------------------------------------
; LISTING 3: Utility Routines
;------------------------------------------------
(defun MAKE_A_LINE (P1 P2 L)
  (entmake (list
    '(0 . "LINE")
    (cons 8 L)
    (cons 10 P1)
    (cons 11 P2)
  ))
)
;------------------------------------------------
(defun GETIT (Prm Def / Gitter TMP)
  (setq Gitter (type Def)
        Gitter
    (cond
      ((= Gitter 'REAL)
        '(getreal 
           (strcat 
              Prm 
              " <" 
              (rtos Def) 
              ">: "))
      )
      ((= Gitter 'INT)
        '(getint 
           (strcat 
              Prm 
              " <" 
              (itoa Def) 
              ">: "))
      )
      ((= Gitter 'STR)
        '(getstring 
           1 
           (strcat 
              Prm 
              " <" 
              Def 
              ">: "))
      )
      (t nil)
    )
  )
  (if GITTER (progn
    (setq TMP (eval GITTER))
    (if TMP TMP Def)
    )
    Def
  )
)
;;----------------------------------------------- EOF
